#define STACK_START	0x20000000

#include "regmap.h"

.text
	.global _start
_start:
	/* delay */
	mov	r0, #0x80000
delay_loop:
	subs	r0, r0, #1
	bne	delay_loop

	/* PAD Control setup (To Support 133MHz DDR) */
	
	ldr	r0, =IOMUXC_BASE_ADDR
	mov	r1, #0
	mov	r2, #0x1000
	str	r1, [r0, #0x26c] /* SDCLK */
	str	r1, [r0, #0x270] /* CAS */
	str	r1, [r0, #0x274] /* RAS */
	str	r2, [r0, #0x27c] /* CS2 (CSD0) */
	str	r1, [r0, #0x284] /* DQM3 */

	/* DQM2-DQM0, SD31-SD0, A25-A0, MA10(0x0288..0x02dc) */
	add	r0, r0, #0x288
	mov	r2, #0x16
pad_loop_:
	str	r1, [r0]
	add	r0, r0, #0x04
	subs	r2, r2, #0x01
	bne	pad_loop_

	/* IPU Setup */
	ldr	r0, =IPU_BASE_ADDR
	mov	r1, #0x40
	str	r1, [r0]

	/* Clock Setup */
	/* RVAL/WVAL for L2 cache memory */
	ldr	r0, =0x515
	ldr	r1, =CLK_CTRL_BASE_ADDR
	str	r0, [r1, #0x10]

	/* default setting
	 *   PLL@ MCU: 399MHz, USB: 240MHz
	 *   CLK@ MCU: 399MHz, HCLK: 133MHz, IPG: 66.5MHz
	 */

	ldr	r0, =CCM_BASE_ADDR
	ldr	r1, [r0, #CCMR]
	and	r1, r1, #0x06
	ldr	r2, =0x064b0b79 /* Default Value */
	orr	r1, r1, r2
	orr	r2, r1, #0x80
	bic	r2, r2, #0x308
	str	r2, [r0, #CCMR]

	ldr	r3, =PDR0_DEFAULT
	str	r3, [r0, #0x04]
	
	ldr	r3, =PDR1_DEFAULT
	str	r3, [r0, #0x08]
	
	and	r2, r2, #0x06
	cmp	r2, #0x4
	beq	_setup_pll_use_CKIH
_setup_pll_use_FPM:
	ldr	r2, =MPCTL_FPM_DEFAULT
	str	r2, [r0, #0x10]

	ldr	r2, =UPCTL_FPM_240
	str	r2, [r0, #0x14]

	ldr	r2, =SPCTL_FPM_294
	str	r2, [r0, #0x18]
	b	_setup_pll_start
_setup_pll_use_CKIH:
	ldr	r2, =MPCTL_CKIH_DEFAULT
	str	r2, [r0, #0x10]

	ldr	r2, =UPCTL_CKIH_240
	str	r2, [r0, #0x14]
	
	ldr	r2, =SPCTL_CKIH_294
	str	r2, [r0, #0x18]
_setup_pll_start:
	str	r1, [r0, #CCMR]
	nop
	nop
	nop
	nop

	mov	sp, #STACK_START 

	mov     r0, #0
	ldr	r1, =__bss_start	/* clear bss */
1:	str	r0, [r1], #4
	cmp	r1, sp
	blo	1b

	bl	main			/* see loader.c */
2:	b	2b

	.global flush_v3
flush_v3:
	mov     r0, #0
	mcr     p15, 0, r0, c7, c7, 0	/* flush v3/v4 cache */
	mcr	p15, 0, r0, c5, c0, 0	/* flush v3 TLB */
	mov	pc, lr

	.global flush_v4
flush_v4:
	mov     r0, #0
	mcr     p15, 0, r0, c7, c7, 0	/* flush v3/v4 cache */
	mcr	p15, 0, r0, c8, c7, 0	/* flush v4 TLB */
	mov	pc, lr

